#define SC_USE_USER_DEFINED_VS_MAIN

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

#ifndef GLES20STD_TEXTURE
#define GLES20STD_TEXTURE

#ifdef GL_ES
#if __VERSION__ >=300
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#define texture2DArray texture
#define texture3D texture
#define textureCube texture
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 0
#define SC_SUPPORT_3D_TEXTURE 0
#endif
#else
#define SC_SUPPORT_2D_ARRAY_TEXTURE 1
#define SC_SUPPORT_3D_TEXTURE 1
#endif

#define CLAMP_UV(uv,uvMin,uvMax,useClampToBorder,borderClampFactor){	float clampedUv=clamp(uv,uvMin,uvMax);	if (useClampToBorder){		borderClampFactor*=step(abs(uv -clampedUv),0.00001);	}	uv=clampedUv;}

#define SOFTWARE_WRAP(uv,softwareWrapMode,useClampToBorder,clampToBorderFactor){	if (softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_EDGE || softwareWrapMode==SOFTWARE_WRAP_MODE_CLAMP_TO_BORDER){		CLAMP_UV(uv,0.0,1.0,useClampToBorder,clampToBorderFactor);	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_REPEAT){		uv=fract(uv);	} else if (softwareWrapMode==SOFTWARE_WRAP_MODE_MIRRORED_REPEAT){		float uvFract=fract(uv);		float uvInt=uv -uvFract;		float uvOdd=step(0.25,fract(uvInt*0.5));		uv=mix(uvFract,1.0 -uvFract,clamp(uvOdd,0.0,1.0));	}}

#ifdef GL_ES
#define QUANTIZE_UV(A) A
#else
#define QUANTIZE_UV(A) A - (fract(A*4096.0)/4096.0);
#endif

#ifdef FRAGMENT_SHADER
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv,mipBias)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv,mipBias)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv,mipBias)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv,mipBias)
#else
#define TEXTURE2D(sampler,uv,mipBias) texture2D(sampler,uv)
#define TEXTURE2DARRAY(sampler,uv,mipBias) texture2DArray(sampler,uv)
#define TEXTURE3D(sampler,uv,mipBias) texture3D(sampler,uv)
#define TEXTURECUBE(sampler,uv,mipBias) textureCube(sampler,uv)
#endif

#define SAMPLE_TEXTURE(tex,sampler,uvIn,useUvTransform,uvTransform,softwareWrapModeU,softwareWrapModeV,useUvMinMax,uvMinMax,useClampToBorder,borderColor,mipBias){	vec2 uvTemp=uvIn;	float clampToBorderFactor=1.0;	SOFTWARE_WRAP(uvTemp.x,softwareWrapModeU,useClampToBorder && !useUvMinMax,clampToBorderFactor);	SOFTWARE_WRAP(uvTemp.y,softwareWrapModeV,useClampToBorder && !useUvMinMax,clampToBorderFactor);	if (useUvMinMax){		CLAMP_UV(uvTemp.x,uvMinMax.x,uvMinMax.z,useClampToBorder,clampToBorderFactor);		CLAMP_UV(uvTemp.y,uvMinMax.y,uvMinMax.w,useClampToBorder,clampToBorderFactor);	}	if (useUvTransform){		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));	}	uvTemp=QUANTIZE_UV(uvTemp);	tex=TEXTURE2D(sampler,uvTemp,mipBias);	if (useClampToBorder){		tex=mix(borderColor,tex,clampToBorderFactor);	}}

#if SC_SUPPORT_2D_ARRAY_TEXTURE
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){	vec2 uvTemp=uvwIn.xy;	if (useUvTransform){		uvTemp=vec2(uvTransform*vec3(uvTemp,1.0));	}	uvTemp=QUANTIZE_UV(uvTemp);	tex=TEXTURE2DARRAY(sampler,vec3(uvTemp,uvwIn.z),mipBias);}
#else
#define SAMPLE_TEXTURE_2D_ARRAY(tex,sampler,uvwIn,useUvTransform,uvTransform,mipBias){	tex=vec4(0.0,0.0,0.0,1.0);}
#endif

#if SC_SUPPORT_3D_TEXTURE
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){	vec3 uvwTemp=uvwIn;	uvwTemp=QUANTIZE_UV(uvwTemp);	tex=TEXTURE3D(sampler,uvwTemp,mipBias);}
#else
#define SAMPLE_TEXTURE_3D(tex,sampler,uvwIn,mipBias){	tex=vec4(0.0,0.0,0.0,1.0);}
#endif

#define SAMPLE_TEXTURE_CUBE(tex,sampler,uvwIn,mipBias){	vec3 uvwTemp=uvwIn;	uvwTemp=QUANTIZE_UV(uvwTemp);	tex=TEXTURECUBE(sampler,uvwTemp,mipBias);}

#ifdef CUSTOM_PREPROCESSOR

#endif

#endif

#ifndef saturate
#define saturate(A) clamp(A,0.0,1.0)
#endif

#ifdef VERTEX_SHADER

void main(void){
	sc_Vertex_t v=sc_LoadVertexAttributes();
	sc_ProcessVertex(v);
}

#endif

#ifdef FRAGMENT_SHADER

uniform sampler2D opacityTex;
uniform mat3 opacityTexTransform;
uniform vec4 opacityTexUvMinMax;
uniform vec4 opacityTexBorderColor;
uniform vec4 opacityTexSize;

#ifndef SC_USE_UV_TRANSFORM_opacityTex
#define SC_USE_UV_TRANSFORM_opacityTex false
#define SC_SOFTWARE_WRAP_MODE_U_opacityTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_SOFTWARE_WRAP_MODE_V_opacityTex SOFTWARE_WRAP_MODE_DISABLED
#define SC_USE_UV_MIN_MAX_opacityTex false
#endif
#ifndef SC_USE_CLAMP_TO_BORDER_opacityTex
#define SC_USE_CLAMP_TO_BORDER_opacityTex false
#endif

#ifndef opacityTexUV
#define opacityTexUV 0
#endif

void main(void){
	vec4 surfaceColor=vec4(1.0);

#ifdef sc_ProjectiveShadowsReceiver
	surfaceColor.rgb*=evaluateShadow();
#endif

#if defined(ENABLE_OPACITY_TEX)
		vec4 opacityTexSample;
	SAMPLE_TEXTURE(opacityTexSample,opacityTex,varTex0,SC_USE_UV_TRANSFORM_opacityTex,opacityTexTransform,SC_SOFTWARE_WRAP_MODE_U_opacityTex,SC_SOFTWARE_WRAP_MODE_V_opacityTex,SC_USE_UV_MIN_MAX_opacityTex,opacityTexUvMinMax,SC_USE_CLAMP_TO_BORDER_opacityTex,opacityTexBorderColor,0.0);;
	surfaceColor=mix(vec4(1.0),surfaceColor,opacityTexSample.r);
#endif

	gl_FragColor=surfaceColor;
}

#endif

#if sc_IsEditor
#error This is an exported shader. Please do not use shaders in Studio that have already been exported to a lens! Only use fresh shaders,presets,or shaders from existing Studio projects!
#endif

/// Exported with Lens Studio 2.3.1.0
